#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_django-excel-to-model
------------

Tests for `django-excel-to-model` models module.
"""

from django.test import TestCase

from django_excel_to_model import models


class TestDjango_excel_to_model(TestCase):

    def setUp(self):
        pass

    def test_something(self):
        pass

    def tearDown(self):
        pass
